@@ -70,6 +70,8 @@ public class Html5TagProvider implements ITagInfoProvider {
 	 * HTML5 Media Tags
 	 */
 	private static final String MEDIA_TAGS = "audio,video,object,source";
+	
+	private static final String SCRIPT_SUPPORTING_TAGS = "script,template";
 
 	public Html5TagProvider() {
 		TagInfo tagInfo = null;
@@ -85,7 +87,7 @@ public class Html5TagProvider implements ITagInfoProvider {
 		tableTags(tagInfo);
 		metadataTags(tagInfo);
 		scriptingTags(tagInfo);
-		INSTANCE2 = new MathMLTagProvider(tagInfo, tagInfoMap);
+		//INSTANCE2 = new MathMLTagProvider(tagInfo, tagInfoMap);
 	}
 	
 	public void embeddedContentTags(TagInfo tagInfo) {
@@ -93,9 +95,11 @@ public class Html5TagProvider implements ITagInfoProvider {
 		// SVG
 		tagInfo = new TagInfo("svg", ContentType.all, BelongsTo.BODY, false,
 				false, false, CloseTag.required, Display.block);
+		tagInfo.defineAllowedChildrenTags("animate,animateMotion,animateTransform,discard,set,desc,title,metadata,linearGradient,radialGradient,pattern,circle,ellipse,line,path,polygon,polyline,rect,defs,g,svg,symbol,use,a,audio,canvas,clipPath,filter,foreignObject,iframe,image,marker,mask,script,style,switch,text,video,view");
 		tagInfo.defineCloseBeforeCopyInsideTags(CLOSE_BEFORE_COPY_INSIDE_TAGS);
-		tagInfo.defineCloseBeforeTags("svg," + CLOSE_BEFORE_TAGS);
+		tagInfo.defineCloseBeforeTags(CLOSE_BEFORE_TAGS);
 		tagInfo.setAssumedNamespace("http://www.w3.org/2000/svg");
+		tagInfo.setAssumedNamespacePrefix("svg");
 		this.put("svg", tagInfo);
 			
 		// MathML
@@ -103,12 +107,13 @@ public class Html5TagProvider implements ITagInfoProvider {
 				false, false, CloseTag.required, Display.block);
 		tagInfo.defineCloseBeforeCopyInsideTags(CLOSE_BEFORE_COPY_INSIDE_TAGS);
 		tagInfo.defineCloseBeforeTags("math,summary,address,label,abbr,acronym,dfn,kbd,samp,var,cite,code,param,xml");
-		tagInfo.defineForbiddenTags("math");
+		//tagInfo.defineForbiddenTags("math");
 		//
 		// We'll add this later - right now it causes more problems than it solves
 		// as there are no tag name clashes between MathML and HTML unlike in SVG.
 		//
 		tagInfo.setAssumedNamespace("http://www.w3.org/1998/Math/MathML");
+		tagInfo.setAssumedNamespacePrefix("mathml");
 		//
 		this.put("math", tagInfo);
 	}
@@ -343,18 +348,21 @@ public class Html5TagProvider implements ITagInfoProvider {
 				false, false, CloseTag.required, Display.block);
 		tagInfo.defineCloseBeforeCopyInsideTags(CLOSE_BEFORE_COPY_INSIDE_TAGS);
 		tagInfo.defineCloseBeforeTags(CLOSE_BEFORE_TAGS);
-		tagInfo.defineAllowedChildrenTags("dt,dd");
+		tagInfo.defineAllowedChildrenTags("dt,dd,div,"+SCRIPT_SUPPORTING_TAGS);
+		tagInfo.setPreferredChildTag("div");
 		this.put("dl", tagInfo);
 
 		tagInfo = new TagInfo("dt", ContentType.all, BelongsTo.BODY, false,
 				false, false, CloseTag.optional, Display.block);
 		tagInfo.defineCloseBeforeTags("dt,dd");
+		tagInfo.defineAllowedChildrenTags(FLOW_TAGS);
 		tagInfo.defineRequiredEnclosingTags("dl");
 		this.put("dt", tagInfo);
 
 		tagInfo = new TagInfo("dd", ContentType.all, BelongsTo.BODY, false,
 				false, false, CloseTag.optional, Display.block);
 		tagInfo.defineCloseBeforeTags("dt,dd");
+		tagInfo.defineAllowedChildrenTags(FLOW_TAGS);
 		tagInfo.defineRequiredEnclosingTags("dl");
 		this.put("dd", tagInfo);
 
