@@ -531,8 +531,7 @@ public class HtmlTokenizer {
         String tagName = transformations.getTagName(originalTagName);
 
         if (tagName != null) {
-            ITagInfoProvider tagInfoProvider = cleaner.getTagInfoProvider();
-            TagInfo tagInfo = tagInfoProvider.getTagInfo(tagName);
+            TagInfo tagInfo = cleaner.getTagInfo(tagName, this.cleanTimeValues);
             if ( (tagInfo == null && !props.isOmitUnknownTags() && props.isTreatUnknownTagsAsContent() && !isReservedTag(tagName) && !props.isNamespacesAware()) ||
                  (tagInfo != null && tagInfo.isDeprecated() && !props.isOmitDeprecatedTags() && props.isTreatDeprecatedTagsAsContent()) ) {
                 content();
@@ -601,8 +600,7 @@ public class HtmlTokenizer {
         }
 
         if (tagName != null) {
-            ITagInfoProvider tagInfoProvider = cleaner.getTagInfoProvider();
-            TagInfo tagInfo = tagInfoProvider.getTagInfo(tagName);
+            TagInfo tagInfo = cleaner.getTagInfo(tagName, this.cleanTimeValues);
         	if ( (tagInfo == null && !props.isOmitUnknownTags() && props.isTreatUnknownTagsAsContent() && !isReservedTag(tagName) &&!props.isNamespacesAware()) ||
                  (tagInfo != null && tagInfo.isDeprecated() && !props.isOmitDeprecatedTags() && props.isTreatDeprecatedTagsAsContent()) ) {
                 content();
@@ -796,7 +794,10 @@ public class HtmlTokenizer {
             saveCurrent();
             go();
         }
-
+        
+        if (props.isDeserializeEntities()){
+        	return Utils.deserializeEntities(result.toString(), props.isRecognizeUnicodeChars());
+        }
 
         return result.toString();
     }
