@@ -55,7 +55,6 @@ import org.htmlcleaner.conditional.TagNodeNameCondition;
  * </p>
  */
 public class TagNode extends TagToken implements HtmlNode {
-    private TagNode parent;
     private final LinkedHashMap<String, String> attributes = new LinkedHashMap<String, String>();
     private final List<BaseToken> children = new ArrayList<BaseToken>();
     private DoctypeToken docType;
@@ -207,7 +206,7 @@ public class TagNode extends TagToken implements HtmlNode {
                 }
     			String keyToSet = key; // the key to set
     			String value = attributes.get(key); // the value to set
-    			value = Utils.deserializeEntities(value, true);
+    			//value = Utils.deserializeEntities(value, true);
 
     			//
     			// Check to see if the key exists in the current attribute set
@@ -281,7 +280,7 @@ public class TagNode extends TagToken implements HtmlNode {
             	// overwrite it.
             	//
             	if (!attributes.containsKey(trim)){
-            		attributes.put(trim, Utils.deserializeEntities(value, true));
+            		attributes.put(trim, value);
             	}
             }
         }
@@ -421,13 +420,6 @@ public class TagNode extends TagToken implements HtmlNode {
         }
     }
 
-    /**
-     * @return Parent of this node, or null if this is the root node.
-     */
-    public TagNode getParent() {
-        return parent;
-    }
-
     public DoctypeToken getDocType() {
         return docType;
     }
