@@ -50,7 +50,8 @@ public class Utils {
 	static final String VALID_XML_IDENTIFIER_START_CHAR_REGEX = "^[:A-Z_a-z\\u00C0\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02ff\\u0370-\\u037d"
 			+ "\\u037f-\\u1fff\\u200c\\u200d\\u2070-\\u218f\\u2c00-\\u2fef\\u3001-\\ud7ff"
 			+ "\\uf900-\\ufdcf\\ufdf0-\\ufffd\\x{10000}-\\x{EFFFF}]";
-	static final Pattern VALID_XML_IDENTIFIER_START_CHAR_PATTERN = Pattern.compile(VALID_XML_IDENTIFIER_START_CHAR_REGEX, Pattern.UNICODE_CHARACTER_CLASS);
+	static final Pattern VALID_XML_IDENTIFIER_START_CHAR_PATTERN = 
+			compileUnicodePattern(VALID_XML_IDENTIFIER_START_CHAR_REGEX);
 	
 	/*
 		The relevant production from the spec is http://www.w3.org/TR/xml/#NT-Name
@@ -58,14 +59,16 @@ public class Utils {
 		NameStartChar ::= ":" | [A-Z] | "_" | [a-z] | [#xC0-#xD6] | [#xD8-#xF6] | [#xF8-#x2FF] | [#x370-#x37D] | [#x37F-#x1FFF] | [#x200C-#x200D] | [#x2070-#x218F] | [#x2C00-#x2FEF] | [#x3001-#xD7FF] | [#xF900-#xFDCF] | [#xFDF0-#xFFFD] | [#x10000-#xEFFFF]
 		NameChar ::= NameStartChar | "-" | "." | [0-9] | #xB7 | [#x0300-#x036F] | [#x203F-#x2040]
 	 */
-	static final String VALID_XML_IDENTIFIER_CHAR_REGEX = "^[:A-Z_a-z\\u00C0\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02ff\\u0370-\\u037d"
+	static final String VALID_XML_IDENTIFIER_CHAR_REGEX = 
+			  "^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02ff\\u0370-\\u037d"
 			+ "\\u037f-\\u1fff\\u200c\\u200d\\u2070-\\u218f\\u2c00-\\u2fef\\u3001-\\ud7ff"
 			+ "\\uf900-\\ufdcf\\ufdf0-\\ufffd\\x{10000}-\\x{EFFFF}]"
-			+ "[:A-Z_a-z\\u00C0\\u00D6\\u00D8-\\u00F6"
+			+ "[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6"
 			+ "\\u00F8-\\u02ff\\u0370-\\u037d\\u037f-\\u1fff\\u200c\\u200d\\u2070-\\u218f"
 			+ "\\u2c00-\\u2fef\\u3001-\\udfff\\uf900-\\ufdcf\\ufdf0-\\ufffd\\-\\.0-9"
 			+ "\\u00b7\\u0300-\\u036f\\u203f-\\u2040]*\\Z";
-	static final Pattern VALID_XML_IDENTIFIER_CHAR_PATTERN = Pattern.compile(VALID_XML_IDENTIFIER_CHAR_REGEX, Pattern.UNICODE_CHARACTER_CLASS);
+	static final Pattern VALID_XML_IDENTIFIER_CHAR_PATTERN = 
+			compileUnicodePattern(VALID_XML_IDENTIFIER_CHAR_REGEX);
 
 	
 	/**
@@ -597,7 +600,7 @@ public class Utils {
     	// Attribute names must consist of one or more characters other than controls, 
     	// U+0020 SPACE, U+0022 ("), U+0027 ('), U+003E (>), U+002F (/), U+003D (=), and noncharacters.
     	String regex = "[\\u0000\\u0020\\u0022\\u0027\\u003E\\u002F\\u003d]";
-    	Pattern pattern = Pattern.compile(regex, Pattern.UNICODE_CHARACTER_CLASS);
+    	Pattern pattern = compileUnicodePattern(regex);
     	final Matcher matcher = pattern.matcher(name);
     	name = matcher.replaceAll("");
     	return name;
@@ -605,7 +608,7 @@ public class Utils {
     
     public static boolean isValidHtmlAttributeName(String name){
     	String regex = "^[^\\u0000\\u0020\\u0022\\u0027\\u003E\\u002F\\u003d]+$";
-    	Pattern pattern = Pattern.compile(regex, Pattern.UNICODE_CHARACTER_CLASS);
+    	Pattern pattern = compileUnicodePattern(regex);
     	final Matcher matcher = pattern.matcher(name);
     	return matcher.find();
     }
@@ -656,8 +659,13 @@ public class Utils {
      */
     public static boolean isValidXmlIdentifier(String s) {
     	if (s == null) return false;
-    	final Matcher matcher = Utils.VALID_XML_IDENTIFIER_CHAR_PATTERN.matcher(s);
-    	return matcher.find();
+    	Matcher matcher = VALID_XML_IDENTIFIER_CHAR_PATTERN.matcher(s);
+		if (matcher.find()){
+			s = null;
+			matcher = null;
+			return true;
+		}
+		return false;
     }
 
     /**
@@ -876,12 +884,22 @@ public class Utils {
     			+ "\\u00F8-\\u02ff\\u0370-\\u037d\\u037f-\\u1fff\\u200c\\u200d\\u2070-\\u218f"
     			+ "\\u2c00-\\u2fef\\u3001-\\udfff\\uf900-\\ufdcf\\ufdf0-\\ufffd\\-\\.0-9"
     			+ "\\u00b7\\u0300-\\u036f\\u203f-\\u2040]";
-    	final Pattern pattern = Pattern.compile(regex_repl, Pattern.UNICODE_CHARACTER_CLASS);
+    	final Pattern pattern = compileUnicodePattern(regex_repl);
     	final Matcher matcher = pattern.matcher(name);
     	name = matcher.replaceAll(replacement);
 
     	return name;
     }
+    
+    
+    private static Pattern compileUnicodePattern(String pattern){
+    	try {
+    		return Pattern.compile(pattern, Pattern.UNICODE_CHARACTER_CLASS);
+    	} catch(IllegalArgumentException ex) {
+    		return Pattern.compile(pattern);
+    	}
+    	
+    }
 
 
     
