@@ -9,14 +9,262 @@ import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.StringWriter;
+import java.util.List;
 
 import javax.xml.parsers.ParserConfigurationException;
 
 import org.junit.Ignore;
 import org.junit.Test;
+import org.w3c.dom.Document;
+import org.w3c.dom.Node;
+import org.w3c.dom.NodeList;
 
 public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
 	
+	// Test for bug 221. Thanks to Simon Urli for contributing the test case.
+	@Test
+    public void parseWithUnicodeChars() throws Exception
+    {
+        CleanerProperties cleanerProperties = new CleanerProperties();
+
+        cleanerProperties.setDeserializeEntities(true);
+        cleanerProperties.setRecognizeUnicodeChars(true);
+        cleanerProperties.setTranslateSpecialEntities(false);
+
+        HtmlCleaner cleaner = new HtmlCleaner(cleanerProperties);
+        TagNode tagNode = cleaner.clean("<?xml version = \"1.0\"?>"
+            + "<div foo=\"&#169;\">&#169;</div>"
+            + "<div foo=\"baz&gt;buz\">baz&gt;buz</div>"
+            + "<div foo=\"baz&buz\">baz&buz</div>");
+        List<? extends TagNode> divList = tagNode.getElementListByName("div", true);
+        assertEquals(3, divList.size());
+
+        assertEquals("©", divList.get(0).getText().toString());
+        assertEquals("©", divList.get(0).getAttributeByName("foo"));
+
+        assertEquals("baz>buz", divList.get(1).getText().toString());
+        assertEquals("baz>buz", divList.get(1).getAttributeByName("foo"));
+
+        assertEquals("baz&buz", divList.get(2).getText().toString());
+        assertEquals("baz&buz", divList.get(2).getAttributeByName("foo"));
+
+        DomSerializer domSerializer = new DomSerializer(cleanerProperties, false);
+        Document document = domSerializer.createDOM(tagNode);
+
+        NodeList nodeList = document.getElementsByTagName("div");
+        assertEquals(3, nodeList.getLength());
+        assertEquals("©", nodeList.item(0).getTextContent());
+        assertEquals("©", nodeList.item(0).getAttributes().getNamedItem("foo").getTextContent());
+
+        assertEquals("baz>buz", nodeList.item(1).getAttributes().getNamedItem("foo").getTextContent());
+
+        assertEquals("baz&buz", nodeList.item(2).getAttributes().getNamedItem("foo").getTextContent());
+
+        // BUG: This is failing with baz&gt;buz
+        assertEquals("baz>buz", nodeList.item(1).getTextContent());
+        // BUG: This is failing with baz&amp;buz
+        assertEquals("baz&buz", nodeList.item(2).getTextContent());
+    }
+	
+	// See Bug 219. Ensure we don't see any odd structure changes
+	// when handling foreign markup.
+	@Test
+	public void unknownNS() throws Exception
+	{
+		String initial = 
+				  "<html xmlns='foo'>"
+                + "<head>"
+                + "</head>"
+                + "<body>"
+                + "<a><br>"
+                + "<h3\">text</h3>"
+                + "<div\">"
+                + "<cite\">link</cite>"
+                + "</div></a>"               
+                + "</body>"
+                + "</html>";
+		String expected = 				  
+				  "<html xmlns=\"foo\">"
+                + "<head>"
+                + "</head>"
+                + "<body>"
+                + "<a><br>"
+                + "<h3>text</h3>"
+                + "<div>"
+                + "<cite>link</cite>"
+                + "</div></br></a>"               
+                + "</body>"
+                + "</html>";
+		assertCleanedHtml(initial, expected);
+	}
+	
+	// See Bug 218. Thanks to Lukas Lehmann and Dennis Ratzke
+	@Test
+	public void structuredListTest() throws Exception
+	{
+		String initial = "<em>*</em>";
+		String expected = "<html><head></head><body><em>*</em></body></html>";
+		assertCleanedHtml(initial, expected);
+		
+		initial = "<label>banana</label>";
+		expected = "<html><head></head><body><label>banana</label></body></html>";
+		assertCleanedHtml(initial, expected);
+		
+		initial = "<label>banana<em>*</em></label>";
+		expected = "<html><head></head><body><label>banana<em>*</em></label></body></html>";
+		assertCleanedHtml(initial, expected);
+		
+		initial = "<label>banana<em>*</em></label><select><option>d</option></select>";
+		expected = "<html><head></head><body><label>banana<em>*</em></label><select><option>d</option></select></body></html>";
+		assertCleanedHtml(initial, expected);
+		
+		initial = "<dl>\n" +
+				"<div>\n" + 
+				"<label>bb<em>*</em></label>\n" + 
+				"<select>\n" + 
+				"<option>d</option>\n" + 
+				"</select>\n" + 
+				"</div>\n" + 
+				"</dl>\n";
+		expected = "<html><head></head><body>" + 
+				"<dl>\n"+
+				"<div>\n" +
+				"<label>bb<em>*</em></label>\n" +
+				"<select>\n" +
+				"<option>d</option>\n" +
+				"</select>\n" +
+				"</div>\n" +
+				"</dl>\n"+
+				"</body></html>";
+		assertCleanedHtml(initial, expected);
+		
+		initial = "<dl><label>banana</label></dl>";
+		expected = "<html><head></head><body><dl><div><label>banana</label></div></dl></body></html>";
+		assertCleanedHtml(initial, expected);
+
+
+		
+	}
+	
+	
+	@Test
+    public void doubleEscapingInAttributesDom() throws Exception
+    {
+        CleanerProperties cleanerProperties = new CleanerProperties();
+        HtmlCleaner cleaner = new HtmlCleaner(cleanerProperties);
+        TagNode tagNode = cleaner.clean("<?xml version = \"1.0\"?><div foo=\"&amp;quot;\"></div>");
+        DomSerializer domSerializer = new DomSerializer(cleanerProperties, true);
+        Document document = domSerializer.createDOM(tagNode);
+
+        Node node = document.getElementsByTagName("div").item(0);
+        assertEquals("&quot;", node.getAttributes().getNamedItem("foo").getTextContent());
+        
+        domSerializer = new DomSerializer(cleanerProperties, false);
+        document = domSerializer.createDOM(tagNode);
+
+        node = document.getElementsByTagName("div").item(0);
+        assertEquals("&amp;quot;", node.getAttributes().getNamedItem("foo").getTextContent());
+    }
+	
+	@Test
+    public void doubleEscapingInAttributesJDom() throws Exception
+    {
+        CleanerProperties cleanerProperties = new CleanerProperties();
+        HtmlCleaner cleaner = new HtmlCleaner(cleanerProperties);
+        TagNode tagNode = cleaner.clean("<?xml version = \"1.0\"?><div foo=\"&amp;quot;\"></div>");
+        JDomSerializer domSerializer = new JDomSerializer(cleanerProperties, true);
+        org.jdom2.Document document = domSerializer.createJDom(tagNode);
+        org.jdom2.Element node = document.getRootElement().getChild("body").getChild("div");
+        assertEquals("&quot;", node.getAttribute("foo").getValue());
+        
+        domSerializer = new JDomSerializer(cleanerProperties, false);
+        document = domSerializer.createJDom(tagNode);
+        node = document.getRootElement().getChild("body").getChild("div");
+        assertEquals("&amp;quot;", node.getAttribute("foo").getValue());
+        
+        cleanerProperties.setDeserializeEntities(true);
+        tagNode = cleaner.clean("<?xml version = \"1.0\"?><div foo=\"&amp;quot;\"></div>");
+        domSerializer = new JDomSerializer(cleanerProperties, false);
+        document = domSerializer.createJDom(tagNode);
+        node = document.getRootElement().getChild("body").getChild("div");
+        assertEquals("&quot;", node.getAttribute("foo").getValue());
+    }
+	
+	@Test
+    public void doubleEscapingInAttributesTagNode() throws Exception
+    {
+        CleanerProperties cleanerProperties = new CleanerProperties();
+        HtmlCleaner cleaner = new HtmlCleaner(cleanerProperties);
+        TagNode tagNode = cleaner.clean("<?xml version = \"1.0\"?><div foo=\"&amp;quot;\"></div>");
+        assertEquals("&amp;quot;", tagNode.findElementHavingAttribute("foo", true).getAttributeByName("foo"));
+        
+        cleanerProperties = new CleanerProperties();
+        cleanerProperties.setDeserializeEntities(true);
+        cleaner = new HtmlCleaner(cleanerProperties);
+        tagNode = cleaner.clean("<?xml version = \"1.0\"?><div foo=\"&amp;quot;\"></div>");
+        assertEquals("&quot;", tagNode.findElementHavingAttribute("foo", true).getAttributeByName("foo"));
+        
+        cleanerProperties.setDeserializeEntities(false);
+        cleaner = new HtmlCleaner(cleanerProperties);
+        tagNode = cleaner.clean("<?xml version = \"1.0\"?><div foo=\"&amp;quot;\"></div>");
+        assertEquals("&amp;quot;", tagNode.findElementHavingAttribute("foo", true).getAttributeByName("foo"));
+    }
+	
+	@Test
+    public void doubleEscapingInAttributesHtml() throws Exception
+    {
+        CleanerProperties cleanerProperties = new CleanerProperties();
+        HtmlCleaner cleaner = new HtmlCleaner(cleanerProperties);
+        cleanerProperties.setDeserializeEntities(true);
+        TagNode tagNode = cleaner.clean("<?xml version = \"1.0\"?><div foo=\"&amp;quot;\"></div>");
+        
+        HtmlSerializer ser = new SimpleHtmlSerializer(cleaner.getProperties());
+        assertTrue(ser.getAsString(tagNode).contains("foo=\"&quot;\""));
+    }
+	
+	// test discussion item
+	@Test
+	public void encodingTest() throws Exception{
+		String html="<div>Today I want to talk about the &amp;lt; character reference.</div>";
+		CleanerProperties props = new CleanerProperties();
+	    props.setOmitHtmlEnvelope(true);
+	    props.setDeserializeEntities(true);
+	    HtmlCleaner cleaner = new HtmlCleaner(props);
+	    TagNode rootTag = cleaner.clean(html);
+	    ContentNode contentNode = (ContentNode)((TagNode)rootTag.getAllChildren().get(0)).getAllChildren().get(0);
+	    assertEquals("Today I want to talk about the &lt; character reference.", contentNode.content);
+	}
+
+	
+	// test discussion item
+	@Test
+	public void encodingTest2() throws Exception{
+		String html="<div>this &amp; that</div>";
+		CleanerProperties props = new CleanerProperties();
+	    props.setOmitHtmlEnvelope(true);
+	    props.setOmitXmlDeclaration(true);
+	    props.setDeserializeEntities(true);
+	    props.setTranslateSpecialEntities(true);
+	    props.setHtmlVersion(5);
+	    HtmlCleaner cleaner = new HtmlCleaner(props);
+	    TagNode rootTag = cleaner.clean(html);
+	    ContentNode contentNode = (ContentNode)((TagNode)rootTag.getAllChildren().get(0)).getAllChildren().get(0);
+	    assertEquals("this & that", contentNode.content);
+		String cleaned = new SimpleHtmlSerializer(cleaner.getProperties(), false).getAsString(cleaner.clean(html));
+	}
+	
+	@Test
+	public void encodingTest3() throws Exception{
+		String html="<div>Today I want to talk about the &amp;lt; character reference.</div>";
+		CleanerProperties props = new CleanerProperties();
+	    props.setOmitHtmlEnvelope(true);
+	    props.setDeserializeEntities(false);
+	    HtmlCleaner cleaner = new HtmlCleaner(props);
+	    TagNode rootTag = cleaner.clean(html);
+	    ContentNode contentNode = (ContentNode)((TagNode)rootTag.getAllChildren().get(0)).getAllChildren().get(0);
+	    assertEquals("Today I want to talk about the &amp;lt; character reference.", contentNode.content);
+	}
+	
 	// test bug #65
 	@Test
 	public void nbspTest() throws Exception{
@@ -182,7 +430,7 @@ public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
 	@Test
 	public void minimalNSTest() throws IOException{
 		String initial = "<html xmlns=\"x\"><ul><a>";
-		String expected = "<html xmlns=\"x\"><head></head><body><a></a><ul></ul></body></html>";
+		String expected = "<html xmlns=\"x\"><head></head><body><ul><a></a></ul></body></html>";
 		assertCleanedHtml(initial, expected);
 	}
 	
@@ -816,8 +1064,16 @@ public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
         String output = serializer.getAsString(cleaned);
         assertEquals(expected, output);	
         
-		initial = "<mtable><mtr></mtr></mtable>";
-		expected = "<html><head /><body><mtable><mtr></mtr></mtable></body></html>";
+		initial = "<math><mtable><mtr></mtr></mtable></math>";
+		expected = "<html><head /><body><math><mtable><mtr /></mtable></math></body></html>";
+		cleaner.getProperties().setAddNewlineToHeadAndBody(false);
+		cleaner.getProperties().setNamespacesAware(true);
+        cleaned = cleaner.clean(initial);
+        output = serializer.getAsString(cleaned);
+        assertEquals(expected, output);	
+        
+		initial = "<math><mi>1</mi><math><mi>2</mi></math></math>";
+		expected = "<html><head /><body><math><mi>1</mi></math><math><mi>2</mi></math></body></html>";
 		cleaner.getProperties().setAddNewlineToHeadAndBody(false);
 		cleaner.getProperties().setNamespacesAware(true);
         cleaned = cleaner.clean(initial);
@@ -830,8 +1086,8 @@ public class HtmlCleanerTest extends AbstractHtmlCleanerTest {
 	 * This is to test issue #131
 	 * @throws IOException
 	 */
-	@Ignore // We should fix this, but it isn't critical
-	@Test
+	//@Ignore // We should fix this, but it isn't critical
+	//@Test
 	public void moveTableContent() throws IOException{
 		String initial = "<table><tbody><h2>hi</h2><tr><td></td></tr></tbody></table>";
 		String expected = "<html><head /><body><h2>hi</h2><table><tbody><tr><td></td></tr></tbody></table></body></html>";
